package com.ruoyi.web.controller.dingding;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.dingtalkcontact_1_0.models.GetUserResponse;
import com.aliyun.dingtalkoauth2_1_0.models.GetUserTokenResponse;
import com.dingtalk.api.response.*;
import com.fasterxml.jackson.databind.JsonNode;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.web.service.SysLoginService;
import com.ruoyi.web.controller.dingding.request.login.AuthLoginRequest;
import com.ruoyi.web.controller.dingding.service.DingAuthServiceInfo;
import com.ruoyi.web.controller.dingding.service.DingThirdTokenService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;

/**
 * <p>DingLoginController 此类用于：钉钉企业内部应用免登（H5微应用）</p>
 * <p>@remark：钉钉企业内部微应用DEMO, 实现了身份验证（免登）功能</p>
 */
@Api(value = "dingAuthController", tags = "钉钉第三方企业应用免登（H5微应用）")
@Controller
@RequestMapping(value = "/auth/ding")
@Slf4j
public class DingThirdAuthController {

    @Resource
    private DingThirdTokenService dingAuthTokenService;

    @Resource
    private DingAuthServiceInfo dingAuthServiceInfo;

    @Autowired
    private SysLoginService loginService;


    @PostMapping(value = "/callback")
    @ResponseBody
    public Map<String, String> callback(@RequestParam String signature,
                                        @RequestParam String timestamp,
                                        @RequestParam String nonce,
                                        @RequestBody JsonNode encryptNode) {
        String encryptMsg = encryptNode.get("encrypt").textValue();
        String plainText = dingAuthServiceInfo.decryptText(signature, timestamp, nonce, encryptMsg);
        JSONObject callBackContent = JSON.parseObject(plainText);

        //进入回调事件分支选择
        Map<String, String> resultMap = dingAuthServiceInfo.caseProcess(callBackContent);
        return resultMap;
    }


    @ApiOperation(value = "第三方企业应用用户登录")
    @PostMapping("/authLogin")
    @ResponseBody
    public AjaxResult authLogin(@RequestBody AuthLoginRequest authLoginRequest) {
        log.info("钉钉用户登录第三方企业应用code:{},corpId:{}",authLoginRequest.getCode(),authLoginRequest.getCorpId());
        //调用接口获取第三方企业应用的access_token，详情请参考获取第三方应用授权企业的accessToken。
//        String corpAccessToken = dingAuthTokenService.getCorpAccessToken(authLoginRequest.getCorpId());
        GetUserTokenResponse userTokenResponse = dingAuthTokenService.getUserAccessToken(authLoginRequest.getCode());
        log.info("用户登录信息corpAccessToken" + JSONObject.toJSONString(userTokenResponse));
        String corpAccessToken = userTokenResponse.getBody().getAccessToken();
        //获取用户userid。
//        OapiV2UserGetuserinfoResponse userUnfo = dingAuthTokenService.getUserUnfo(authLoginRequest.getCode(), corpAccessToken);
//        log.info("用户登录信息" + JSONObject.toJSONString(userUnfo));
//        //调用接口获取用户的userid，详情请参考通过免登码获取用户信息。
//        OapiV2UserGetResponse authUser = dingAuthTokenService.getAuthUser(corpAccessToken, userUnfo.getResult().getUserid());
//
//        log.info("用户登录详细信息" + JSONObject.toJSONString(authUser.getResult()));

        GetUserResponse addressBookUserInfo = dingAuthTokenService.getAddressBookUserInfo(corpAccessToken, "me");
        log.info("用户通讯录信息" + JSONObject.toJSONString(addressBookUserInfo));
        //获取用户详情。
        SysUser sysUser = dingAuthServiceInfo.getUserByUserId(addressBookUserInfo.getBody().getMobile());
        if (StringUtils.isNull(sysUser)) {
            // todo 生成用户信息进行登录
            dingAuthServiceInfo.addUserAndOrg2(addressBookUserInfo.getBody(),authLoginRequest.getCorpId());
        }
        String token = loginService.authDingThirdLogin(addressBookUserInfo.getBody().getMobile(), authLoginRequest.getCorpId());
        return AjaxResult.success(token);
    }

    @ApiOperation(value = "第三方企业应用授权")
    @PostMapping("/cropLogin")
    @ResponseBody
    public AjaxResult cropLogin(@RequestBody AuthLoginRequest authLoginRequest) {
        log.info("钉钉用户登录第三方企业应用code:{},corpId:{}",authLoginRequest.getCode(),authLoginRequest.getCorpId());
        //调用接口获取第三方企业应用的access_token，详情请参考获取第三方应用授权企业的accessToken。
        String corpAccessToken = dingAuthTokenService.getCorpAccessToken(authLoginRequest.getCorpId());
//        GetUserTokenResponse userTokenResponse = dingAuthTokenService.getUserAccessToken(authLoginRequest.getCode());
        log.info("用户登录信息corpAccessToken" +corpAccessToken);
        //获取用户userid。
        OapiV2UserGetuserinfoResponse userUnfo = dingAuthTokenService.getUserUnfo(authLoginRequest.getCode(), corpAccessToken);
        log.info("用户登录信息" + JSONObject.toJSONString(userUnfo));
        //调用接口获取用户的userid，详情请参考通过免登码获取用户信息。
        OapiV2UserGetResponse authUser = dingAuthTokenService.getAuthUser(corpAccessToken, userUnfo.getResult().getUserid());

        log.info("用户登录详细信息" + JSONObject.toJSONString(authUser.getResult()));

        GetUserResponse addressBookUserInfo = dingAuthTokenService.getAddressBookUserInfo(corpAccessToken, "me");
        log.info("用户通讯录信息" + JSONObject.toJSONString(addressBookUserInfo));
        //获取用户详情。
        SysUser sysUser = dingAuthServiceInfo.getUserByUserId(addressBookUserInfo.getBody().getMobile());
        if (StringUtils.isNull(sysUser)) {
            // todo 生成用户信息进行登录
            dingAuthServiceInfo.addUserAndOrg2(addressBookUserInfo.getBody(),authLoginRequest.getCorpId());
        }
        String token = loginService.authDingThirdLogin(addressBookUserInfo.getBody().getMobile(), authLoginRequest.getCorpId());
        return AjaxResult.success(token);
    }
}